//----------------------------------------
// Routines lecture MIDI
// pour piloter le SN76489
//----------------------------------------
                  

//-----------------------------------------
// Handler CC MIDI
//----------------------------------------
void MyHandleCtrl(byte channel, byte number, byte value) {
	if (channel > 3)
		return;
	// Test du numero de controleur
	switch(number) {
		// Si molette de modulation
		// alors on active le lfo pour le bon canal
		case MODWHEEL:
			// On teste le mode en cours
			// Monophonique, polyphonique ou unison
			switch (ModeMidi) {
				case MONO :						// Monophonique
					if (value >=64)				// Si molette active
						lfo[channel-1]=1;		// on toggle lfo
					else
						lfo[channel-1]=9;		// sinon on prepare sa remise a zero en le mettant a 9
					break;
				case UNISON :
				case POLY :						// Polyphonique
					if (channel!=1)				// On doit etre sur le canal 1 midi en mode POLY
						return;					// Si cest pas le cas on sort
					if (value >=64) {			// Si molette active
						lfo[0]=1;				// on toggle les lfo des 3 canaux du SN76489
						lfo[1]=1;				// pour les 3 canaux
						lfo[2]=1;
					}
					else {						
						lfo[0]=9;				// sinon on prepare sa remise a zero en le mettant a 9
						lfo[1]=9;				// et ca pour les 3 canaux
						lfo[2]=9;
					}
					break;
			}
			break;
	}
}


//----------------------------- 
// HANDLER NOTE OFF MIDI -> CV
//----------------------------- 
void  MyHandleNoteOff(byte channel, byte mynote, byte velocity) {
int i;
	// Si canal en dehors de 1,2,3 ou 10 on sort
	if (channel>3 && channel !=10)
		return;

	// Si Mode UNISON ou POLY
	// et qu'on est pas sur premier canal ou canalbruit
	if (ModeMidi==UNISON || ModeMidi==POLY) {
		if (channel !=1 && channel !=10) 
			return;
	}		
	
	// Tranformation canal midi en SN_channel
	if (channel == 10)			// SI canal 10 alors utilisation du canal SN 3
		SN_channel=3;
	else						// Sinon SN_channel = canal midi diminue de 1
		SN_channel=channel-1;


	// Si canal de bruit
	// alors on mute le canal 3 et on sort
	if (SN_channel==3 && MidinoteNumber[SN_channel]==mynote) {
		UpdateAmplitude(3,0,0);
		return;
	}

	//
	// Arrive ici nous sommes forcement sur un canal musique
	//
	

	// Si note <48 on ne joue pas	
	if (mynote < 48)
		return;

	switch (ModeMidi) {
		case MONO:
			// Si note-off sur derniere note jouee
			// alors on mute le canal concerné
			if (MidinoteNumber[SN_channel]==mynote) {
				if (stateRelease[SN_channel])						// On verifie que l'enveloppe est bien declencheee
					stateRelease[SN_channel]=RELEASE_IN_PROGRESS;	// On initialise la MAJ du Release
				// UpdateAmplitude(SN_channel,0,0);
			}

			break;
		case UNISON:
			// Si note-off sur derniere note jouee
			// alors on mute les canaux
			if (MidinoteNumber[SN_channel]==mynote) {
				for (i=0; i<3; i++) {
					if (stateRelease[i])							// On verifie que l'enveloppe est bien declencheee
						stateRelease[i]=RELEASE_IN_PROGRESS;		// On initialise la MAJ du Release
					MidinoteNumber[i] = 0;						// RAZ note
					MidinoteVelocity[i] = 0;					// RAZ velocite
					// UpdateAmplitude(i,0,0);					// RAZ amplitude
				}
			}
			break;
		case POLY:
			// on cherche le canal qui est occupe par le note off en cours
			polyTimerCnt++;														// une nouvelle note jouee
			for (i=0; i<3; i++) {
				// on cherche un canal occupe par la note en cours de note-off
				if (MidinoteNumber[i] == mynote) {								// si on trouve une note en cours
					if (stateRelease[i])							// On verifie que l'enveloppe est bien declencheee
						stateRelease[i]=RELEASE_IN_PROGRESS;		// On initialise la MAJ du Release
					MidinoteNumber[i] = 0;										// memorisation de la note (le canal nest plus libre)
					MidinoteVelocity[i] = 0;									// memorisation velocite
					MidinotePlay[i] = pgm_read_word_near( MidiToReg+mynote);	// memorisation frequence
					//UpdateAmplitude(i,0,0);
					//PlayNote(i);												// on joue la note sur le canal qui est libre
					break;														// on sort de la boucle
				}
			}
			// Si on arrive ici, on a recu un note-off pour une note qui n'est pas jouee
			// donc il ne se passe rien...
			break;
	}
	if (SN_channel < 4) {
	}

}

//---------------------------- 
// HANDLER NOTE ON MIDI -> CV
//---------------------------- 
void MyHandleNoteOn(byte channel, byte mynote, byte velocity) {
int i;
int searchold;
	// Si velocity == 0 alors on a recu un note off
	//---------------------------------------------
	if (velocity==0) {
		MyHandleNoteOff(channel,mynote,velocity);
	return;
	}

	// Si canal en dehors de 1,2,3 ou 10 on sort
	if (channel>3 && channel !=10)
		return;

	// Si Mode UNISON
	// et qu'on est pas sur premier canal ou canalbruit
	if (ModeMidi==UNISON || ModeMidi==POLY) {
		if (channel !=1 && channel !=10) 
			return;
	}

	
	// Tranformation canal midi en SN_channel
	if (channel == 10)			// Si canal 10 alors utilisation du canal SN 3
		SN_channel=3;
	else						// Sinon SN_channel = canal midi diminue de 1
		SN_channel=channel-1;
		
		
	// Si note <48 et canaux musique on ne joue pas	
	if (mynote < 48 && SN_channel<3)
		return;
		
		
	// Si canal de bruit
	if (SN_channel==3) {
		MidinoteNumber[SN_channel] = mynote;
		MidinoteVelocity[SN_channel] = velocity;
		MidinotePlay[SN_channel] = pgm_read_word_near( MidiToReg+mynote);
		PlayNoise();
		return;
	}

	//MidinoteBend[SN_channel]=0;				// RAZ du pitchbend pour le canal en cours
	//------------------------------------------------------------
	// Arrive ici on est forcement sur un canal musique (0,1 ou 2)
	// le jeu peut etre MONO, UNISON ou POLY
	//------------------------------------------------------------
	switch (ModeMidi) {
		// Mode monophonique 3 parties
		//----------------------------
		case MONO:
			v_decrement[SN_channel] = (float)velocity / (float)v_release;	// Valeur a decrementer pour le release a chaque decrementation
			v_velocityInit[SN_channel] = (float)velocity;						// Velocity en float
			stateRelease[SN_channel] = RELEASE_INIT;								// Prevoir futur Release pour ce canal
			MidinoteNumber[SN_channel] = mynote;								// memorisation de la note
			MidinoteVelocity[SN_channel] = velocity;							// memorisation de la velocite
			MidinotePlay[SN_channel] = pgm_read_word_near( MidiToReg+mynote);	// memorisation de la frequence
			PlayNote(SN_channel);												// on joue la note sur le canal en cours
			break;
		// Mode unison 1 partie mais 3 canaux detunes
		//-------------------------------------------
		case UNISON:
			// On joue la note en cours sur les 3 canaux
			// la procedure UpdateUnison se charge du detuning
			// dans la boucle principale
			
			for (i=0; i<3; i++) {
				v_decrement[i] = (float)velocity / (float)v_release;	// Valeur a decrementer pour le release a chaque decrementation
				v_velocityInit[i] = (float)velocity;						// Velocity en float
				stateRelease[i] = RELEASE_INIT;							// Prevoir futur Release pour ce canal 
				MidinoteNumber[i] = mynote;										// memorisation de la note						
				MidinoteVelocity[i] = velocity;									// memorisation de la velocite
				MidinotePlay[i] = pgm_read_word_near( MidiToReg+mynote);		// memorisation de la frequence
				PlayNote(i);													// on joue la note sur le canal defini par i
			}
			break;
		// Mode polyphonique 3 notes et 1 partie
		//--------------------------------------
		case POLY:
			polyTimerCnt++;														// une nouvelle note doit etre jouee
			// On joue la note en cours sur un canal libre
			// normalement elle ne peut pas etre jouee 2 fois de suite
			// sans avoir un note off de relachement entre les 2 note-on
			for (i=0; i<3; i++) {
				// on cherche un canal libre
				// si il est libre on a 0 dans le numero de note midi
				if (MidinoteNumber[i] == 0) {									// si canal libre (la note est a 0)
					v_decrement[i] = (float)velocity / (float)v_release;			// Valeur a decrementer pour le release a chaque decrementation
					v_velocityInit[i] = (float)velocity;						// Velocity en float
					stateRelease[i] = RELEASE_INIT;									// Prevoir futur Release pour ce canal
					MidinoteNumber[i] = mynote;									// memorisation de la note (le canal nest plus libre)
					MidinoteVelocity[i] = velocity;								// memorisation velocite
					MidinoteTimer[i]=polyTimerCnt;								// memorisation ordre note
					MidinotePlay[i] = pgm_read_word_near( MidiToReg+mynote);	// memorisation frequence
					PlayNote(i);												// on joue la note sur le canal qui est libre
					break;														// on sort de la boucle
				}
			}
				
			// Si on arrive ici et que i=MAXPOLY c'est qu'il n'y a aucun canal de disponible
			// on remplacera la plus ancienne note
			// pour cela on cherche la note avec le timer de declenchement le plus ancien
			// (le plus petit en fait...)
			if (i >= MAXPOLY) {
				searchold=0;												// init canal note la plus ancienne sur le canal 0
				for (i=1;i<3;i++) {											// on cherche si les autres canaux sont plus vieux
					if (MidinoteTimer[i] < MidinoteTimer[searchold]) 		// si canal en cours plus vieux que canal precedent test
						searchold=i;										// si oui on memorise le numero du canal le plus vieux
				}
				// arrive ici searchold contient le numero de canal le plus vieux
				// on va donc initialiser et jouer ce canal
				v_decrement[searchold] = (float)velocity / (float)v_release;			// Valeur a decrementer pour le release a chaque decrementation
				v_velocityInit[searchold] = (float)velocity;						// Velocity en float
				stateRelease[searchold] = RELEASE_INIT;									// Prevoir futur Release pour ce canal
				MidinoteNumber[searchold] = mynote;									// memorisation de la note (le canal nest plus libre)
				MidinoteVelocity[searchold] = velocity;								// memorisation velocite
				MidinoteTimer[searchold]=polyTimerCnt;								// memorisation ordre note
				MidinotePlay[searchold] = pgm_read_word_near( MidiToReg+mynote);	// memorisation frequence
				PlayNote(searchold);												// on joue la note sur le canal qui est libre
			}
			break;
	}
}

//-------------------
// HANDLER PITCH-BEND
//-------------------
void MyHandlePitchBend (byte channel, int value) {
int i;
	// Si canal en dehors de 1,2,3 ou 10 on sort
	if (channel>3 && channel !=10)
		return;

	// Si Mode UNISON
	// et qu'on est pas sur premier canal ou canalbruit
	if (ModeMidi==UNISON || ModeMidi==POLY) {
		if (channel !=1 && channel !=10) 
			return;
	}

	
	// Tranformation canal midi en SN_channel
	if (channel == 10)			// Si canal 10 alors utilisation du canal SN 3
		SN_channel=3;
	else						//
		// bend = (value>>4-512);
		 bend = (value>>4);
	if (bend>512) bend = 512;
	if (bend<-512) bend = -512;
	switch (ModeMidi) {
		case UNISON:
			MidinoteBend[SN_channel]=bend*-1;
			RefreshFrequency(0);
			RefreshFrequency(1);
			RefreshFrequency(2);
			break;
			
		case MONO:
			MidinoteBend[SN_channel]=bend*-1;
			RefreshFrequency(SN_channel);
			break;
		case POLY:
			for (i=0;i<3;i++) {
				MidinoteBend[i]=bend*-1;
				RefreshFrequency(i);
			}
		
			break;
			
	}

	// MidinoteBend[SN_channel]=bend*-1;
	// RefreshFrequency(SN_channel);
}


