//---------------------------------------------------------------
// Procédures pour programmer le chip SN76489
// Permettent de controler l'amplitude, la fréquence et le bruit
//---------------------------------------------------------------


//------------------------------
// Coupe toutes les attenuateurs
// plus de son sur aucun canal
//------------------------------
void SoundOffSN76489(uint8_t nochips) {
	UpdateSN76489(0x9f,nochips);	
	UpdateSN76489(0xbf,nochips);
	UpdateSN76489(0xdf,nochips);	
	UpdateSN76489(0xff,nochips);	
}

//----------------------------------
// Musique de demarrage
//----------------------------------
void InitSN76489(uint8_t nochips) {
int i;
		UpdateFrequency(0,358,nochips);
		UpdateAmplitude(0,127,nochips);
		delay(200);
		UpdateFrequency(0,284,nochips);
		delay(200);
		UpdateFrequency(1,239,nochips);
		UpdateAmplitude(1,127,nochips);
		delay(200);
		UpdateFrequency(2,179,nochips);
		UpdateAmplitude(2,127,nochips);
		delay(200);
		for (i=127; i>=0; i--) {
				UpdateAmplitude(0,i,nochips);
				UpdateAmplitude(1,i,nochips);
				UpdateAmplitude(2,i,nochips);
				delay(10);		
		}		
		
}


//---------------------------------------------
// Procedure mise à jour d'un octet sur SN76489
//---------------------------------------------
void UpdateSN76489(uint8_t dataSN, uint8_t nochips ) {
uint8_t CS_PIN_SN;
uint8_t bufdataSN;
uint8_t inPORT;
	CS_PIN_SN = nochips + PIN_CS1;
	bufdataSN=dataSN<<2;					// Recuperation 6 bits de poids faible des datas
	inPORT=PORTD&3;							// Masquage pour lire le PORTD sur le 2 premiers bits uniquement
	PORTD=bufdataSN+inPORT;					// reecrire des 2 premiers bits et 6 bits de poids faibles data
	inPORT=PORTC&252;						// recuperation de PORTC sauf 2 premiers bits remis a zero
	bufdataSN=dataSN>>6;					// recuperation des 2 derniers bits des datas en decalant de 6
	PORTC=inPORT+bufdataSN;					// reecrire les 2 premiers bits de PORTC sans changer le reste
	delayMicroseconds(DELAY_WRITE_SN);
	digitalWrite(CS_PIN_SN, LOW);
	delayMicroseconds(DELAY_WRITE_SN);
	digitalWrite(CS_PIN_SN, HIGH);
	delayMicroseconds(DELAY_WRITE_SN);
}

//-----------------------------------------------
// Mise a jour des attenuateurs
// channel :   de 0 à 2 pour musique, 3 pour noise
// amplitude : Attenuation 0 à 127 (en midi) transformée de 0 a 15 (0 = son au maxi, 15=mini)
// chipNumber : no de chip SN76489 (si plusieurs...)
//-----------------------------------------------
void UpdateAmplitude(uint8_t channel, uint8_t amplitude, uint8_t chipNumber ) {
uint8_t n_channel,n_amplitude,DataOut;
	// Si derniere MAJ du chip identique pour le canal
	// alors on sort
	if (last_dataSN_Amplitude[channel] == amplitude )
		return;
	last_dataSN_Amplitude[channel] = amplitude;
	n_channel = channel << 5;							// conversion banque format SN
	n_amplitude = 15 -(amplitude >> 3);					// conversion velocity midi en format SN
	DataOut = B10010000 + n_channel + n_amplitude;		// calcul octet a ecrit pour MAJ amplitude
	UpdateSN76489(DataOut,chipNumber);						// maj SN76489
}	


//-----------------------------------------------
// Mise a jour du canal noise
// Type noise contient le genre de bruit. 8 possibles
// nochips : no de chip SN76489 (si plusieurs...)
//-----------------------------------------------
void UpdateNoise(uint8_t typenoise, uint8_t chipNumber ) {
uint8_t DataOut;
	DataOut = B11100000+typenoise;							// calcul octet a ecrit pour MAJ amplitude
	UpdateSN76489(DataOut,chipNumber);						// maj SN76489
}	


//----------------------------------
// Update de la frequence d'un canal
//----------------------------------
void UpdateFrequency(uint8_t channel, uint16_t dataSN, byte chipNumber ) {
uint8_t n_channel,n_amplitude,DataOut;
int16_t n_dataSN;
	// Si derniere MAJ du chip identique pour le canal
	// alors on sort
	if (last_dataSN_Frequency[channel] == dataSN )
		return;
	last_dataSN_Frequency[channel] = dataSN;
	n_channel = channel << 5;				// conversion canal format SN
	if (dataSN < 0 )
		dataSN = 0;
	if (dataSN > 1023 )
		dataSN = 1023;
	n_dataSN = dataSN % 16;							// garder que les 4 premiers bits
	DataOut = B10000000 | n_channel | n_dataSN;		// calcul 1er octet  pour MAJ frequency
	UpdateSN76489(DataOut,chipNumber);					// ecrire cet octet dans SN76489
	DataOut = (dataSN >> 4) & B00111111	;			// calcul 2eme octet pour MAJ frequency
	UpdateSN76489(DataOut,chipNumber);					// ecrire ce deuxieme octet dans SN76489
}


//-----------------------------------------------
// Procedure de refresh de la frequence si besoin
//-----------------------------------------------
void RefreshFrequency(uint8_t SN_channel) {
int16_t vfreq;
	switch (ModeMidi) {
		case MONO :
		case POLY:
			if (MidinoteBend[SN_channel]>=0)
				vfreq=MidinotePlay[SN_channel]+vunison+((float)MidinoteBend[SN_channel]/512.0*MidinotePlay[SN_channel]+vlfo[SN_channel]);
			else
				vfreq=MidinotePlay[SN_channel]+vunison+((float)MidinotePlay[SN_channel]/1024.0*MidinoteBend[SN_channel]+vlfo[SN_channel]);
			break;
		case UNISON:
			if (MidinoteBend[0]>=0)
				vfreq=MidinotePlay[0]+vunison+((float)MidinoteBend[0]/512.0*MidinotePlay[0]+vlfo[0]);
			else
				vfreq=MidinotePlay[0]+vunison+((float)MidinotePlay[0]/1024.0*MidinoteBend[0]+vlfo[0]);
			break;
	}
	if (vfreq<0 || vfreq >1023)			// si on sort des frequence autorise
		return;							// on laisse tomber update
	UpdateFrequency(SN_channel,vfreq,0);	// sinon on met a jour le canal
}

//------------------------------------------------------
// Procedure pour jouer une note sur un canal du SN76489
//------------------------------------------------------
void PlayNote(uint8_t SN_channel) {
	switch (ModeMidi) {
		case MONO :
		case UNISON:
		case POLY:
			RefreshFrequency(SN_channel);
			//UpdateFrequency(SN_channel,MidinotePlay[SN_channel],0);
			UpdateAmplitude(SN_channel,MidinoteVelocity[SN_channel],0);
			break;
	}
	return;
}

//---------------------------------------------
// Procedure pour faire jouer le canal de bruit
//---------------------------------------------
void PlayNoise()
{
uint8_t tnoise;
	// SI note <36 ou > 51 on sort
	if (MidinoteNumber[3] <36 || MidinoteNumber[3] >43)
		return;
	tnoise=MidinoteNumber[3]-36;
	UpdateNoise(tnoise,0);
	UpdateAmplitude(3,MidinoteVelocity[3],0);
	return;
}

