//----------------------------------------------------------------------------------
// Programme de pilotage d'un SN76489 avec un arduino
//----------------------------------------------------------------------------------
// 
// Entree en midi
// Modes disponibles
//		MONO : 3 voies monophoniques + 1 noise canaux 1 a 4
//		UNISON : 1 voies en unison (3 registres sons stackees et detunes) canal 1 + 1 noise canal 4
// 		POLY :   1 voies en polyphonique (3 notes  possible en meme temps) canal 1 + 1 noise canal 4
//----------------------------------------------------------------------------------
// Auteur.: Frederic ABIGNOLI
// Date...: 13/02/17
//----------------------------------------------------------------------------------
// Version :
//		1.0 (13/02/17) : 
//				projet initial note on/noteoff midi sur canaux 1 a 3  midi + noise sur canal 10
//
// 		1.1 (03/03/19) : 
//				- interrupteur de mode (Mono/Poly/Unison
//				- tuning
//				- enveloppe release (juste release pour enveloppe)
//				- LFO
//					o- vitesse
//					o- profondeur
//		1.2 (13/12/20)
//				- utilisation des environnements récent arduino et librairie midi récente
//				- correction du pitch-bend pour que ca fonctionne avec nouvelle environnement
//				- ajout ligne de creation instance midi pour lib MIDI recente
//----------------------------------------------------------------------------------



#define DELAY_WRITE_SN	50			// delay en µs pour variation etat CS
									// 25µS OK jusqua 1mhz, pour 500Khz il faut mettre 50µS
									// sinon ca plante le chip
									
#define PIN_D0_SN 		2			// Pin D0 SN76489
#define PIN_D1_SN		3			// Pin D1 SN76489
#define PIN_D2_SN		4			// Pin D2 SN76489
#define PIN_D3_SN		5			// Pin D3 SN76489
#define PIN_D4_SN		6			// Pin D4 SN76489
#define PIN_D5_SN		7			// Pin D5 SN76489
#define PIN_CLOCK		9			// Pin clock SN76489

#define PIN_D6_SN		A0			// Pin D6 SN76489
#define PIN_D7_SN		A1			// Pin D7 SN76489
#define PIN_CS1 		A2			// Chips select SN76489
#define PIN_MODE 		A3			// Selection mode 3 voies, Poly et Unison
#define PIN_LFOSPEED	A4			// Reglage frequence LFO
#define PIN_LFODEPTH	A5			// Reglage profondeur LFO
#define PIN_RANGE 		A6			// Reglage range (vitesse horloge 4 à 1 mhz)
#define PIN_RELEASE 	A7			// Reglage Release



#if defined(ARDUINO) && ARDUINO >= 100
#define printByte(args)  write(args);
#else
#define printByte(args)  print(args,BYTE);
#endif

// Mode de jeu en MIDI
#define	MONO	1		// monophonique 3 parties
#define UNISON	2		// monophonique 1 partie 3 osc en unison
#define POLY	3		// polyphonique 3 notes 
#define MAXPOLY 3		// Maximum de note en polyphonique
#define MODWHEEL	1	// Controleur midi modulation wheel
uint8_t	ModeMidi;		// Mode midi actuel (mono, poly ou unison)
uint8_t LastMode;		// Dernier mode midi en cours


// LFO
uint16_t lfofreq = 0;   		// lfo period
uint16_t lfodepth = 127;  		// lfo wave amplitude
uint8_t lfotimer = 0;   		// lfo value, counts from 0 to lfofreq
uint16_t lfoval = 0;    		// amount to add to output pitch
uint16_t lfodelta = 0;			// gigue du lfo
long lastlfocnt=0;				// Compteur pour recalcul lfo
uint8_t RefreshRateLfo = 0; 	// Rafraichissement  et recalcul en ms du lfo
int vlfo[] = {0,0,0,0};			// Decalage frequence du au lfo (en negatif ou positif par rapport a la frequence centrale)
uint8_t lfo[]={0,0,0,0};		// Toggle lgo par canal (via modwheel);



uint8_t CountNote;     				// Nombre de note appuyees sur le clavier a un instant "T"

uint16_t MidinotePlay[] = {0,0,0,0};			// Buffer valeur note en registre SN76489 en cours
uint16_t MidinoteVelocity[] = {0,0,0,0};		// Buffer velocity note en cours
uint16_t MidinoteNumber[] = {0,0,0,0};			// Buffer numero note midi
uint32_t MidinoteTimer[] = {0,0,0,0};			// Buffer Timer pour le slide

// Optimisation des transferts
// on memorise les derniers valeurs envoyee
// pour les données de frequences et d'amplitudes
uint16_t last_dataSN_Frequency[] = {0,0,0,0};			// Buffer derniere frequence calculee par canal
uint16_t last_dataSN_Amplitude[] = {255,255,255,255};	// Buffer derniere amplitude calcullee par canal

int16_t MidinoteBend[] = {0,0,0,0};				// Buffer numero note midi

int vunison;									// Decalage unison
uint32_t polyTimerCnt=0;
uint16_t unison_cnt=0;							// Compteur modification unison
uint16_t CALC_UNISON_CNT=3000;					// Comparaison recalcul unison
uint8_t vchannel=0;								// Channel unison en cours (pour MAJ unison)

#define CALC_READ_POT 1000						// Frequence lecture des potentiometres et boutons
#define MAX_ITEMS_READ 5						// Nombre maxi d'items a lire (boutons et potards)
uint8_t items_cnt=0;							// Item courant a lire
uint16_t read_cnt=0;							// Compteur lecture item

int16_t bend=0;
int16_t pbend=0;

byte SN_channel=1;								// numero de canal SN76489

float v_decrement[] = {0,0,0,0};				// Decrement du release
float v_velocityInit[] = {0,0,0,0};				// Velocity en float

#define RELEASE_NO_SOUND 0
#define RELEASE_INIT 1
#define RELEASE_IN_PROGRESS 2
uint8_t stateRelease[]  = {0,0,0,0};			// State du release par canal (0=NoSound, 1=noteOn, 2=release en cours)
#define MULT_RELEASE 	4 						// coeficient multiplicateur pour longueur release
int v_release;									// Temps release en ms (valeur potard x MULT_release)


unsigned long nexttime;

#include "SN76489.h"
#include <MIDI.h>

MIDI_CREATE_DEFAULT_INSTANCE();

//---------------------------------
// SETUP DES VARIABLES
// Preparation des PINS
// Initialisation des interruptions
//---------------------------------
void setup() {
	pinMode(PIN_D0_SN, OUTPUT);		// Sortie pour bit 0 data SN76489
	pinMode(PIN_D1_SN, OUTPUT);		// Sortie pour bit 1 data SN76489
	pinMode(PIN_D2_SN, OUTPUT);		// Sortie pour bit 2 data SN76489
	pinMode(PIN_D3_SN, OUTPUT);		// Sortie pour bit 3 data SN76489
	pinMode(PIN_D4_SN, OUTPUT);		// Sortie pour bit 4 data SN76489
	pinMode(PIN_D5_SN, OUTPUT);		// Sortie pour bit 5 data SN76489
	pinMode(PIN_D6_SN, OUTPUT);		// Sortie pour bit 6 data SN76489
	pinMode(PIN_D7_SN, OUTPUT);		// Sortie pour bit 7 data SN76489
	pinMode(PIN_CLOCK, OUTPUT);		// Sortie clock
	pinMode(PIN_CS1, OUTPUT);		// Sortie CS chip no1
	pinMode(PIN_CS1, OUTPUT);		// Sortie CS chip no2
	pinMode(PIN_MODE, INPUT);		// Entree selection de mode jeu avec interrupteur
	pinMode(PIN_LFOSPEED, INPUT);	// Entree lfo speed
	pinMode(PIN_LFODEPTH, INPUT);	// Entree lfo depth
	pinMode(PIN_RANGE, INPUT);		// Entree selection vitesse horloge
	pinMode(PIN_RELEASE, INPUT);	// Entree potard release enveloppe
	
	
	// Generation d'une clock sur la pin 9
	TCCR1A &= ~((1<<COM1A1) | (1<<COM1A0) | (1<<WGM11) | (1<<WGM10));			// Reset TCCR1A
	TCCR1B &= ~((1<<WGM13) | (1<<WGM12) | (1<<CS12) | (1<<CS11) | (1<<CS10));	// Reset TCCR1B
	TCCR1A |= (0<<COM1A1) | (1<<COM1A0);			// Toggle OC1A/OC1B on Compare Match
	TCCR1A |= (0<<WGM11) | (0<<WGM10);				//
	TCCR1B |= (0<<WGM13) | (1<<WGM12);				// CTC Mode
	TCCR1B |= (0<<CS12) | (0<<CS11) | (1<<CS10);	// No prescaling
	OCR1A = 1;										// Clockgen : 1=4 Mhz, 3=2Mhz, 7=1Mhz
	delay(100);
	SoundOffSN76489(0);

	//InitSN76489(0);						// Musique d'accueil :)
	MIDI.begin(MIDI_CHANNEL_OMNI);          // demarre le midi en mode OMNI
	CountNote=0;                            // Compteur de note actives
	SoundOffSN76489(0);
	
	// Initialisation Handler Midi
	//----------------------------
	MIDI.setHandleNoteOn(MyHandleNoteOn);       	// handler note on 
	MIDI.setHandleNoteOff(MyHandleNoteOff);     	// handler note off
	MIDI.setHandleControlChange(MyHandleCtrl);  	// handler CC
	MIDI.setHandlePitchBend (MyHandlePitchBend);// handler pitch-bend
	delay(1000);
	ModeMidi=UNISON;
	LastMode=ModeMidi;
	nexttime=millis()+MULT_RELEASE;
	RefreshRateLfo=10;				// Initialisation Refresh Lfo
}

void loop() {
int i,j;
uint8_t b,c,d;
	MIDI.read();				// lecture midi
	update_unison();			// MAJ oscillateurs unisons
	read_buttons_and_pots();	// Lecture boutons et potards
	update_enveloppe();			// MAJ amplitude
	update_lfo();
}